@@ -19,12 +19,28 @@ RFC2822_DATE_FORMAT     = '%a, %d %b %Y'
 RFC822_DATE_FORMAT      = '%a, %d %b %y'
 # 2012-06-13
 ISO8601_DATE_FORMAT     = '%Y-%m-%d'
+# All date formats
+ALL_DATE_FORMATS        = (
+    RFC2822_DATE_FORMAT,
+    RFC822_DATE_FORMAT,
+    ISO8601_DATE_FORMAT,
+)
 
 # Precalculated timestamps
-TIME_MINUTE             = 60
-TIME_HOUR               = 3600
-TIME_DAY                = 86400
-TIME_WEEK               = 604800
+TIME_MINUTE = 60
+TIME_HOUR   = 3600
+TIME_DAY    = 86400
+TIME_WEEK   = 604800
+
+
+def _total_seconds(t):
+    '''
+    Takes a `datetime.timedelta` object and returns the delta in seconds.
+    '''
+    return sum([
+        int(t.days * 86400 + t.seconds),
+        int(round(t.microseconds / 1000000.0))
+    ])
 
 
 def _to_datetime(t):
@@ -33,29 +49,25 @@ def _to_datetime(t):
     :class:`datetime.datetime` object.
     '''
 
-    if isinstance(t, float) or \
-        isinstance(t, int) or \
-        isinstance(t, long):
-        return datetime.datetime.fromtimestamp(float(t))
+    if isinstance(t, (float, int, long)):
+        return datetime.datetime.fromtimestamp(t).replace(microsecond=0)
 
     elif isinstance(t, basestring):
-        for format in (
-            RFC2822_DATETIME_FORMAT,
-            RFC822_DATETIME_FORMAT,
-            ISO8601_DATETIME_FORMAT,
-            ):
+        for date_format in ALL_DATE_FORMATS:
             try:
-                return datetime.datetime.strptime(t, format)
+                d = datetime.datetime.strptime(t, date_format)
+                return d.replace(microsecond=0)
             except ValueError:
                 pass
 
         raise ValueError('Format not supported')
 
     elif isinstance(t, datetime.datetime):
-        return t
+        return t.replace(microsecond=0)
 
     elif isinstance(t, datetime.date):
-        return datetime.datetime.combine(t, datetime.time(0, 0))
+        d = datetime.datetime.combine(t, datetime.time(0, 0))
+        return d.replace(microsecond=0)
 
     else:
         raise TypeError
@@ -67,17 +79,11 @@ def _to_date(t):
     :class:`datetime.date` object.
     '''
 
-    if isinstance(t, float) or \
-        isinstance(t, int) or \
-        isinstance(t, long):
-        return datetime.date.fromtimestamp(float(t))
+    if isinstance(t, (float, int, long)):
+        return datetime.date.fromtimestamp(t)
 
     elif isinstance(t, basestring):
-        for format in (
-            RFC2822_DATE_FORMAT,
-            RFC822_DATE_FORMAT,
-            ISO8601_DATE_FORMAT,
-            ):
+        for date_format in ALL_DATE_FORMATS:
             try:
                 return datetime.datetime.strptime(t, format).date()
             except ValueError:
@@ -117,7 +123,7 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
 
     # The datetime module includes milliseconds with float precision. Floats
     # will give unexpected results here, so we round the value here
-    total = math.ceil(diff.total_seconds())
+    total = math.ceil(_total_seconds(diff))
     total_abs = abs(total)
 
     if total_abs < TIME_DAY:
@@ -133,7 +139,8 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
                 _multi(
                     _(u'%d second'),
                     _(u'%d seconds'),
-                seconds) % (seconds,),
+                    seconds
+                ) % (seconds,),
                 0,
             )
         elif total_abs < TIME_MINUTE * 2 and words:
@@ -144,12 +151,14 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
 
         elif total_abs < TIME_HOUR:
             minutes, seconds = divmod(total_abs, TIME_MINUTE)
-            if total < 0: seconds *= -1
+            if total < 0:
+                seconds *= -1
             return (
                 _multi(
                     _(u'%d minute'),
                     _(u'%d minutes'),
-                minutes) % (minutes,),
+                    minutes
+                ) % (minutes,),
                 seconds,
             )
 
@@ -161,13 +170,15 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
 
         else:
             hours, seconds = divmod(total_abs, TIME_HOUR)
-            if total < 0: seconds *= -1
+            if total < 0:
+                seconds *= -1
 
             return (
                 _multi(
                     _(u'%d hour'),
                     _(u'%d hours'),
-                hours) % (hours,),
+                    hours
+                ) % (hours,),
                 seconds,
             )
 
@@ -179,12 +190,14 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
 
     elif total_abs < TIME_WEEK:
         days, seconds = divmod(total_abs, TIME_DAY)
-        if total < 0: seconds *= -1
+        if total < 0:
+            seconds *= -1
         return (
             _multi(
                 _(u'%d day'),
                 _(u'%d days'),
-            days) % (days,),
+                days
+            ) % (days,),
             seconds,
         )
 
@@ -202,12 +215,14 @@ def delta(t1, t2, words=True, justnow=datetime.timedelta(seconds=10)):
 
     else:
         weeks, seconds = divmod(total_abs, TIME_WEEK)
-        if total < 0: seconds *= -1
+        if total < 0:
+            seconds *= -1
         return (
             _multi(
                 _(u'%d week'),
                 _(u'%d weeks'),
-            weeks) % (weeks,),
+                weeks
+            ) % (weeks,),
             seconds,
         )
 
@@ -228,7 +243,7 @@ def day(t, now=None, format='%B %d'):
     t1 = _to_date(t)
     t2 = _to_date(now or datetime.datetime.now())
     diff = t1 - t2
-    secs = diff.total_seconds()
+    secs = _total_seconds(diff)
     days = abs(diff.days)
 
     if days == 0:
@@ -280,10 +295,12 @@ def duration(t, now=None, precision=1, pad=u', ', words=None,
 
     result, remains = delta(t1, t2, words=words, justnow=justnow)
     if result in (
-        _(u'just now'),
-        _(u'yesterday'), _(u'tomorrow'),
-        _(u'last week'), _(u'next week'),
-        ):
+            _(u'just now'),
+            _(u'yesterday'),
+            _(u'tomorrow'),
+            _(u'last week'),
+            _(u'next week'),
+    ):
         return result
 
     elif precision > 1 and remains:
@@ -318,8 +335,8 @@ def compress(t, sign=False, pad=u''):
 
     if isinstance(t, datetime.timedelta):
         seconds = t.seconds + (t.days * 86400)
-    elif isinstance(t, float) or isinstance(t, int) or isinstance(t, long):
-        seconds = long(t)
+    elif isinstance(t, (float, int, long)):
+        seconds = long(round(t))
     else:
         return compress(datetime.datetime.now() - _to_datetime(t), sign, pad)
 
